from datetime import datetime
from snakypy import printer, pick
from snakypy.ansi import FG
from snakypy.file import create as snakypy_file_create
from os import remove as os_remove
from os.path import join as os_join
from zshpower.config import package
from zshpower.config.base import Base
from subprocess import check_output
from contextlib import suppress as contextlib_suppress
from zshpower.utils.check import checking_init
from shutil import which as shutil_which
from shutil import copyfile as shutil_copyfile
from shutil import rmtree as shutil_rmtree
from zshpower.utils.process import reload_zsh
from zshpower.utils.catch import read_zshrc_omz
from zshpower.utils.shift import change_theme_in_zshrc, rm_source_zshrc


def rm_config_package(home):
    with contextlib_suppress(Exception):
        os_remove(os_join(home, f".{package.info['pkg_name']}"))
    if shutil_which("pip") is not None:
        check_output(
            f'pip uninstall {package.info["name"]} -y',
            shell=True,
            universal_newlines=True,
        )


class UninstallCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def main(self):
        checking_init(self.HOME)

        if not read_zshrc_omz(self.zsh_rc):
            title = f"Do you want to uninstall {package.info['name']}?"
            options = ["Yes", "No"]
            reply = pick(title, options, colorful=True, index=True)
            if reply is None or reply[0] == 1:
                printer("Whew! Thanks! :)", foreground=FG.GREEN)
                exit(0)
            rm_config_package(self.HOME)
            rm_source_zshrc(self.zsh_rc)
        else:
            title = f"What did you want to uninstall?"
            options = [
                f"{package.info['name']}",
                f"{package.info['name']} and Oh My ZSH",
                "Cancel",
            ]
            reply = pick(title, options, colorful=True, index=True)

            if reply is None or reply[0] == 2:
                printer("Whew! Thanks! :)", foreground=FG.GREEN)
                exit(0)

            with contextlib_suppress(Exception):
                os_remove(self.theme_file)

            rm_config_package(self.HOME)

            change_theme_in_zshrc(self.zsh_rc, "robbyrussell")

            if reply[0] == 1:
                shutil_rmtree(self.omz_root, ignore_errors=True)
                with contextlib_suppress(Exception):
                    shutil_copyfile(
                        self.zsh_rc, f"{self.zsh_rc}-D{datetime.today().isoformat()}"
                    )
                with contextlib_suppress(Exception):
                    os_remove(self.zsh_rc)

            snakypy_file_create("", f"{self.HOME}/.zshrc", force=True)

        reload_zsh()
        printer("Uninstall process finished.", foreground=FG.FINISH)



from snakypy.console import cmd as cmd_snakypy
from snakypy.path import create as snakypy_path_create
from snakypy.file import create as snakypy_file_create
from os import remove as os_remove
from datetime import datetime
from re import sub as re_sub, M as re_m
from shutil import copyfile as shutil_copyfile
from zipfile import ZipFile
from os.path import join, exists, isdir, isfile
from sys import platform
from tomlkit import parse as toml_parse, dumps as toml_dumps
from snakypy import printer, FG
from zshpower.utils.catch import read_zshrc_omz, plugins_current_zshrc, read_zshrc


def create_config(content, file_path, force=False):
    if not exists(file_path) or force:
        parsed_toml = toml_parse(content)
        write_toml = toml_dumps(parsed_toml)
        snakypy_file_create(write_toml, file_path, force=force)
        return True
    return


def create_zshrc(content, zshrc):
    if exists(zshrc):
        if not read_zshrc_omz(zshrc):
            shutil_copyfile(zshrc, f"{zshrc}-{datetime.today().isoformat()}")
            snakypy_file_create(content, zshrc, force=True)
            return True
    elif not exists(zshrc):
        snakypy_file_create(content, zshrc)
        return True
    return


def change_theme_in_zshrc(zshrc, theme_name):
    if read_zshrc_omz(zshrc):
        current_zshrc = read_zshrc(zshrc)
        current_theme = read_zshrc_omz(zshrc)[1]
        new_theme = f'ZSH_THEME="{theme_name}"'
        new_zsh_rc = re_sub(rf"{current_theme}", new_theme, current_zshrc, flags=re_m)
        snakypy_file_create(new_zsh_rc, zshrc, force=True)
        return True
    return


def omz_install(omz_root):
    omz_github = "https://github.com/ohmyzsh/ohmyzsh.git"
    cmd_line = f"git clone {omz_github} {omz_root}"
    try:
        if not exists(omz_root):
            printer("Install Oh My ZSH...", foreground=FG.QUESTION)
            cmd_snakypy(cmd_line, verbose=True)
            printer(
                "Oh My ZSH installation process finished.", foreground=FG.FINISH,
            )

    except Exception:
        raise Exception("Error downloading Oh My ZSH. Aborted!")


def omz_install_plugins(omz_root, plugins):
    try:
        url_master = "https://github.com/zsh-users"
        for plugin in plugins:
            path = join(omz_root, f"custom/plugins/{plugin}")
            clone = f"git clone {url_master}/{plugin}.git {path}"
            if not isdir(path):
                printer(f"Install plugins {plugin}...", foreground=FG.QUESTION)
                cmd_snakypy(clone, verbose=True)
                printer(f"Plugin {plugin} task finished!", foreground=FG.FINISH)
    except Exception:
        raise Exception(f"There was an error installing the plugin")


def install_fonts(home, force=False):
    url = "https://github.com/snakypy/snakypy-static"
    base_url = f"blob/master/zshpower/fonts/fonts.zip?raw=true"
    font_name = "DejaVu Sans Mono Nerd Font"
    fonts_dir = join(home, f".fonts")
    snakypy_path_create(fonts_dir)
    curl_output = join(home, "zshpower__font.zip")

    if platform.startswith("linux"):
        try:
            if (
                not isfile(join(fonts_dir, "DejaVu Sans Mono Nerd Font Complete.ttf"))
                or force
            ):
                printer(
                    f'Please wait, downloading the "{font_name}" font and'
                    "installing...",
                    foreground=FG.QUESTION,
                )
                cmd_line = f"curl -L {join(url, base_url)} -o {curl_output}"
                cmd_snakypy(cmd_line, verbose=True)

                with ZipFile(curl_output, "r") as zip_ref:
                    zip_ref.extractall(fonts_dir)
                    os_remove(curl_output)
                    printer("Done!", foreground=FG.FINISH)
                return True
            return
        except Exception as err:
            raise Exception(f'Error downloading font "{font_name}"', err)
    return


def add_plugins_zshrc(zshrc):
    plugins = (
        "python",
        "django",
        "pip",
        "pep8",
        "autopep8",
        "zsh-syntax-highlighting",
        "zsh-autosuggestions",
    )
    current = plugins_current_zshrc(zshrc)
    new_plugins = []
    for plugin in plugins:
        if plugin not in current:
            new_plugins.append(plugin)

    if len(new_plugins) > 0:
        current_zshrc = read_zshrc(zshrc)
        plugins = f'plugins=({" ".join(current)} {" ".join(new_plugins)})'
        new_zsh_rc = re_sub(rf"^plugins=\(.*", plugins, current_zshrc, flags=re_m)
        snakypy_file_create(new_zsh_rc, zshrc, force=True)
        return new_zsh_rc
    return


def rm_source_zshrc(zshrc):
    current_zshrc = read_zshrc(zshrc)
    line_rm = "source\\ \\$HOME/.zshpower"
    new_zshrc = re_sub(rf"{line_rm}", "", current_zshrc, flags=re_m)
    snakypy_file_create(new_zshrc, zshrc, force=True)



from .lib.utils import Color, symbol_ssh


class Command:
    def __init__(self, config):
        self.config = config
        self.cmd_symbol = symbol_ssh(config["command"]["symbol"], "> ")
        self.cmd_color = config["command"]["color"]
        self.cmd_new_line = config["command"]["new_line"]["enable"]

    def __str__(self, jump_line="\n"):
        if not self.cmd_new_line:
            jump_line = ""
        cmd_export = (
            f"{jump_line}{Color(self.cmd_color)}" f"{self.cmd_symbol}{Color().NONE}"
        )
        return str(cmd_export)



from os.path import join, isdir
from os import getcwd, environ
from .lib.utils import Color
from .lib.utils import symbol_ssh, git_status, element_spacing, separator


class Git(Color):
    def __init__(self, config, icon_space=" "):
        super().__init__()
        self.config = config
        self.git_enable = config["git"]["enable"]
        self.git_symbol = symbol_ssh(config["git"]["symbol"], "git:")
        self.git_color_symbol = config["git"]["color"]["symbol"]
        self.git_branch_color = config["git"]["branch"]["color"]
        self.git_prefix_color = config["git"]["prefix"]["color"]
        self.git_prefix_text = element_spacing(config["git"]["prefix"]["text"])
        self.git_symbol_enable = config["git"]["status"]["symbols"]["enable"]
        self.icons = {
            "A": [
                f"{Color('green')}"
                f"{symbol_ssh(config['git']['status']['symbol']['added'], '')}"
                f"{Color().NONE}",
                f"{Color('green')}+{icon_space}{Color().NONE}",
            ],
            # "AM": [
            #     f"{Color('white')}"
            #     f"{symbol_ssh(config_file['git']['status']['symbol']['changed'], '')}"
            #     f"{Color().NONE}",
            #     f"{Color('white')}#{icon_space}{Color().NONE}",
            # ],
            "M": [
                f"{Color('blue')}"
                f"{symbol_ssh(config['git']['status']['symbol']['modified'], '')}"
                f"{Color().NONE}",
                f"{Color('blue')}#{icon_space}{Color().NONE}",
            ],
            "D": [
                f"{Color('red')}"
                f"{symbol_ssh(config['git']['status']['symbol']['deleted'], '')}"
                f"{Color().NONE}",
                f"{Color('red')}x{icon_space}{Color().NONE}",
            ],
            "??": [
                f"{Color('yellow')}"
                f"{symbol_ssh(config['git']['status']['symbol']['untracked'], '')}"
                f"{Color().NONE}",
                f"{Color('yellow')}?{icon_space}{Color().NONE}",
            ],
            "R": [
                f"{Color('magenta')}"
                f"{symbol_ssh(config['git']['status']['symbol']['renamed'], '')}"
                f"{Color().NONE}",
                f"{Color('magenta')}->{icon_space}{Color().NONE}",
            ],
            "UU": [
                f"{Color('red')}"
                f"{symbol_ssh(config['git']['status']['symbol']['conflicts'], '')}"
                f"{Color().NONE}",
                f"{Color('red')}!={icon_space}{Color().NONE}",
            ],
            "AH": [
                f"{Color('blue')}"
                f"{symbol_ssh(config['git']['status']['symbol']['ahead'], '')}"
                f"{Color().NONE}",
                f"{Color('blue')}^{icon_space}{Color().NONE}",
            ],
            "BH": [
                f"{Color('magenta')}"
                f"{symbol_ssh(config['git']['status']['symbol']['behind'], '')}"
                f"{Color().NONE}",
                f"{Color('magenta')}_{icon_space}{Color().NONE}",
            ],
            "DG": [
                f"{Color('yellow')}"
                f"{symbol_ssh(config['git']['status']['symbol']['diverged'], '')}"
                f"{Color().NONE}",
                f"{Color('yellow')}<->{icon_space}{Color().NONE}",
            ],
            "C": [
                f"{Color('yellow')}"
                f"{symbol_ssh(config['git']['status']['symbol']['copied'], '')}"
                f"{Color().NONE}",
                f"{Color('yellow')}**{icon_space}{Color().NONE}",
            ],
            "U": [
                f"{Color('magenta')}"
                f"{symbol_ssh(config['git']['status']['symbol']['unmerged'], '')}"
                f"{Color().NONE}",
                f"{Color('magenta')}={icon_space}{Color().NONE}",
            ],
            "CL": [
                f"{Color('green')}"
                f"{symbol_ssh(config['git']['status']['symbol']['clean'], '')}"
                f"{Color().NONE}",
                f"{Color('green')}~{icon_space}{Color().NONE}",
            ],
        }
        self.icons["UD"] = self.icons["UU"]

    def __str__(self):
        if isdir(join(getcwd(), ".git")):
            status_git = git_status(porcelain=True)
            status_git_text = git_status()
            branch_current = git_status(branch=True)
            branch_formated = (
                f"{Color(self.git_prefix_color)}"
                f"{self.git_prefix_text}{Color().NONE}"
                f"{Color(self.git_color_symbol)}"
                f"{self.git_symbol}{Color().NONE}"
                f"{Color(self.git_branch_color)}"
                f"{branch_current}"
                f"{Color().NONE}"
            )
            status_current = []
            status_icons = []
            if "??" in status_git:
                status_current.append("??")
            if "D" in status_git:
                status_current.append("D")
            if "R" in status_git:
                status_current.append("R")
            if "A" in status_git:
                status_current.append("A")
            # if "AM" in status_git:
            #     status_current.append("AM")
            if "M" in status_git:
                status_current.append("M")
            if "UU" in status_git:
                status_current.append("UU")
            if "U" in status_git:
                status_current.append("U")
            if "UD" in status_git:
                status_current.append("UD")
            if "C" in status_git:
                status_current.append("C")
            if "ahead" in status_git_text:
                status_current.append("AH")
            if "behind" in status_git_text:
                status_current.append("BH")
            if "diverged" in status_git_text:
                status_current.append("DG")
            if len(status_git) == 0:
                status_current.append("CL")

            for item in status_current:
                if "SSH_CONNECTION" not in environ:
                    if self.git_symbol_enable:
                        status_icons.append(f"{self.icons[item][0]}")
                    else:
                        status_icons.append(f"{self.icons[item][1]}")
                else:
                    status_icons.append(f"{self.icons[item][1]}")

            # status = f'( {" ".join(sorted(status_icons)).strip()} )'
            # if len(status_current) == 1:
            status = "".join(sorted(status_icons))

            return f"{separator(self.config)}{branch_formated} {status}"
        return ""



from os import geteuid as os_geteuid
from os import environ as os_environ
from zshpower.utils.catch import current_user
from .lib.utils import Color


class Username(Color):
    def __init__(self, config):
        super().__init__()
        self.username_enable = config["username"]["enable"]
        self.username_color = config["username"]["color"]
        if os_geteuid() == 0:
            self.username_color = "red"

    def __str__(self, space_elem=" "):
        if "SSH_CONNECTION" in os_environ or self.username_enable or os_geteuid() == 0:
            user = current_user()
            username_export = (
                f"{Color(self.username_color)}{user}{space_elem}{Color().NONE}"
            )
            return str(username_export)
        return ""



"""CLI - Command Line Interface"""
from snakypy.utils.decorators import only_for_linux
from zshpower.commands.init import InitCommand
from zshpower.commands.activate import ActivateCommand
from zshpower.commands.config import ConfigCommand
from zshpower.commands.deactivate import DeactivateCommand
from zshpower.commands.reset import ResetCommand
from zshpower.commands.uninstall import UninstallCommand
from zshpower.commands.credits import CreditsCommand
from zshpower.utils.decorators import assign_cli
from zshpower.utils.catch import arguments
from zshpower import HOME


# Get arguments Docopt
args = arguments()


@assign_cli(args, "init")
def run_init():
    InitCommand(HOME).main(args, reload=True, message=True)


@assign_cli(args, "config")
def run_config():
    ConfigCommand(HOME).main(args)


@assign_cli(args, "activate")
def run_activate():
    ActivateCommand(HOME).main()


@assign_cli(args, "deactivate")
def run_deactivate():
    DeactivateCommand(HOME).main(args)


@assign_cli(args, "reset")
def run_reset():
    ResetCommand(HOME).main()


@assign_cli(args, "uninstall")
def run_uninstall():
    UninstallCommand(HOME).main()


@assign_cli(args, "--credits")
def run_credits():
    CreditsCommand().main()


@only_for_linux
def main():
    run_init()
    run_config()
    run_activate()
    run_deactivate()
    run_reset()
    run_uninstall()
    run_credits()



from snakypy.ansi import FG, NONE
from zshpower.config import package

dt_omz = f"{FG.MAGENTA}robbyrussell{NONE}"
omz = f"{FG.MAGENTA}Oh My ZSH{NONE}"

options = f"""
{FG.MAGENTA}Welcome to the {package.info["pkg_name"]} options menu.{NONE}

Usage:
    {package.info['executable']} init [--omz]
    {package.info['executable']} config (--open | --view)
    {package.info['executable']} activate
    {package.info['executable']} deactivate [--theme=<name>]
    {package.info['executable']} reset
    {package.info['executable']} uninstall
    {package.info['executable']} --help
    {package.info['executable']} --version
    {package.info['executable']} --credits

Arguments:
    {FG.CYAN}init{NONE} ---------- Installs the {package.info["name"]} settings.
    {FG.CYAN}activate{NONE} ------ Activate the {package.info["name"]} theme. ({omz}).
    {FG.CYAN}deactivate{NONE} ---- Deactivate the {package.info["name"]} theme and go back to the default. ({omz}).
    {FG.CYAN}reset{NONE} --------- Reset to default settings.
    {FG.CYAN}uninstall{NONE} ----- Uninstall the package {package.info["name"]}.
    {FG.CYAN}config{NONE} -------- The easiest way to edit and view the settings is through this option.

Options:
    {FG.CYAN}--help{NONE} --------- Show this screen.
    {FG.CYAN}--omz{NONE} ---------- Use this option if you want to use {package.info["name"]} with Oh My ZSH.
    {FG.CYAN}--open{NONE} --------- Open the configuration file in edit mode and perform the automatic update
                     when you exit.
    {FG.CYAN}--view{NONE} --------- View the configuration file on the terminal.
    {FG.CYAN}--theme=<name>{NONE} - Get the name of a theme available on Oh My ZSH [Default: {dt_omz}].
    {FG.CYAN}--version{NONE} ------ Show version.
    {FG.CYAN}--credits{NONE} ------ Show credits.
"""



from subprocess import call as subprocess_call
from subprocess import Popen as subprocess_popen
from zshpower.utils.catch import current_user, current_shell
from snakypy import printer
from snakypy.ansi import FG


def reload_zsh():
    subprocess_call("exec zsh", shell=True)


def bash_command(cmd):
    subprocess_popen(["/bin/bash", "-c", cmd])


def change_shell():
    if current_shell()[0] != "zsh":
        try:
            subprocess_call(f"chsh -s $(which zsh) {current_user()}", shell=True)
        except KeyboardInterrupt:
            printer("Canceled by user", foreground=FG.WARNING)



from snakypy import printer
from snakypy.ansi import FG
from zshpower.config.base import Base
from zshpower.utils.process import reload_zsh
from zshpower.utils.shift import change_theme_in_zshrc
from zshpower.utils.catch import read_zshrc_omz
from zshpower.utils.check import checking_init


class DeactivateCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def main(self, arguments, *, theme_name="robbyrussell"):
        checking_init(self.HOME)

        if read_zshrc_omz(self.zsh_rc):
            if not read_zshrc_omz(self.zsh_rc)[0] == "zshpower":
                printer("Already disabled. Nothing to do.", foreground=FG.GREEN)
                exit(0)
            if not arguments["--theme"]:
                change_theme_in_zshrc(self.zsh_rc, theme_name)
            else:
                change_theme_in_zshrc(self.zsh_rc, arguments["--theme"])
            printer("Deactivation process finish.", foreground=FG.FINISH)
            reload_zsh()
            return True
        return printer(
            "You are not using Oh My ZSH to run this command.", foreground=FG.WARNING
        )



import os
from subprocess import check_output
# from snakypy import FG
# from snakypy.ansi import NONE as s_none


# class Color:
#     """Colors bash"""
#
#     NONE = s_none
#
#     def __init__(self, set_color="none"):
#         self.set_color = set_color
#         self.color = {"green": FG.GREEN,
#                       "red": FG.RED,
#                       "blue": FG.BLUE,
#                       "yellow": FG.YELLOW,
#                       "cyan": FG.CYAN,
#                       "magenta": FG.MAGENTA,
#                       "white": FG.WHITE,
#                       "black": FG.BLACK,
#                       "none": ""}
#
#     def __str__(self):
#         return self.color.get(self.set_color, "")


# class Color:
#     """Color application compatible only with Oh My ZSH."""
#     NONE = "%{$reset_color%}"
#
#     def __init__(self, set_color=None):
#         self.color = f"%{{$fg[{set_color}]%}}"
#
#     def __str__(self):
#         return self.color

class Color:
    """Color application compatible only ZSH."""
    NONE = "%f"

    def __init__(self, set_color=""):
        self.color = f"%F{{{set_color}}}"
        if set_color == "white":
            self.color = ""

    def __str__(self):
        return self.color


def symbol_ssh(symbol1, symbol2, spacing=" "):
    # TODO: Add icon in SSH by options true/false - Next version
    if symbol1 != "":
        symbol1 += spacing
    if "SSH_CONNECTION" in os.environ:
        symbol1 = symbol2
    return symbol1


def abspath_link():
    cwd = check_output("pwd -L", shell=True, universal_newlines=True).strip()
    return cwd


def git_status(*, porcelain=False, branch=False):
    porcelain_set = "--porcelain" if porcelain else ""
    branch_set = "--branch" if branch else ""
    if porcelain and not branch:
        status = check_output(f"git status {porcelain_set}",
                              shell=True, universal_newlines=True).split()
    elif branch and not porcelain:
        data = check_output(f"git status {branch_set}",
                            shell=True, universal_newlines=True).split()
        if data[0] == "HEAD":
            status = f"{Color('red')}HEAD{Color().NONE}"
        else:
            status = data[2]
    else:
        status = check_output(f"git status {porcelain_set} {branch_set}",
                              shell=True, universal_newlines=True).split()
    return status


def separator(config, spacing=" "):
    sep = config["general"]["separator"]["element"]
    sep += spacing if sep != "" else sep
    sep_color = config["general"]["separator"]["color"]
    separator_style = f"{Color(sep_color)}{sep}{Color().NONE}"
    return separator_style


def element_spacing(element, spacing=" "):
    if element != "":
        element += spacing
    return element



from time import strftime
from .lib.utils import Color
from .lib.utils import symbol_ssh


class Timer(Color):
    def __init__(self, config):
        super().__init__()
        self.timer_enable = config["timer"]["enable"]
        self.timer_symbol = symbol_ssh(config["timer"]["symbol"], "T:")
        self.timer_color = config["timer"]["color"]
        self.timer_seconds_enable = config["timer"]["seconds"]["enable"]

    def __str__(self):
        if self.timer_enable:
            c = Color()
            get_timer = str(strftime("%H:%M"))
            if self.timer_seconds_enable:
                get_timer = str(strftime("%H:%M:%S"))
            timer = (
                f"{Color(self.timer_color)}{self.timer_symbol}" f"{get_timer}{c.NONE}"
            )
            return f"{timer.strip()}"
        return ""



from zshpower import __name__

info = {
    "name": "ZSHPower",
    "description": "ZSHPower is a theme for ZSH with a manager.",
    "pkg_name": __name__,
    "executable": __name__,
    "home_page": "https://github.com/snakypy/zshpower",
    "organization_name": "Snakypy",
    "author": {
        "name": "William C. Canin",
        "email": "william.costa.canin@gmail.com",
        "website": "https://williamcanin.github.io",
        "github": "https://github.com/williamcanin",
    },
    "credence": [
        {
            "my_name": "William C. Canin",
            "email": "william.costa.canin@gmail.com",
            "website": "http://williamcanin.me",
            "locale": "Brazil - SP",
        }
    ],
}



from os import environ as os_environ
from .lib.utils import Color, separator, symbol_ssh, element_spacing


def get_virtualenv_name():
    if "VIRTUAL_ENV" in os_environ:
        venv_name = os_environ["VIRTUAL_ENV"].split("/")[-1]
        # Treatment for virtual machines created with Poetry
        venv_name = "-".join(venv_name.split("-")[:-2])
        return venv_name
    return ""


class Virtualenv(Color):
    def __init__(self, config):
        super().__init__()
        self.config = config
        self.venv_enable = config["virtualenv"]["enable"]
        self.venv_symbol = symbol_ssh(config["virtualenv"]["symbol"], "")
        self.venv_involved = config["virtualenv"]["involved"]
        self.venv_color = config["virtualenv"]["color"]
        self.venv_prefix_color = config["virtualenv"]["prefix"]["color"]
        self.venv_prefix_text = element_spacing(config["virtualenv"]["prefix"]["text"])
        self.venv_name_enable = config["virtualenv"]["name"]["normal"]["enable"]
        self.venv_name_text = config["virtualenv"]["name"]["text"]

    def __str__(self, space_elem=" "):
        involved_prefix = ""
        involved_suffix = ""

        if "VIRTUAL_ENV" in os_environ and self.venv_enable:
            env_prefix = (
                f"{Color(self.venv_prefix_color)}"
                f"{self.venv_prefix_text}{Color().NONE}"
            )
            if len(self.venv_involved) == 2:
                involved_prefix = self.venv_involved[0]
                involved_suffix = self.venv_involved[1]
            if self.venv_name_enable:
                virtualenv = (
                    f"{separator(self.config)}{env_prefix}{Color(self.venv_color)}"
                    f"{self.venv_symbol}"
                    f"{involved_prefix}{get_virtualenv_name()}{involved_suffix}"
                    f"{space_elem}{Color().NONE}"
                )
            else:
                virtualenv = (
                    f"{separator(self.config)}{env_prefix}{Color(self.venv_color)}"
                    f"{self.venv_symbol}"
                    f"{involved_prefix}{self.venv_name_text}{involved_suffix}"
                    f"{space_elem}{Color().NONE}"
                )
            return str(virtualenv)
        return ""



from zshpower.config import package

author = package.info["author"]["name"]
website_author = package.info["author"]["website"]
email = package.info["author"]["email"]

content = f"""#!/usr/bin/env zsh

# Information:
# ******************************************************************************
# Theme: {package.info["name"]}
# Description: Call script to start {package.info["name"]}.
# Author: {author}, {website_author}


export PATH="$PATH:$HOME/.local/bin"
function set_zshpower () {{
    PROMPT="$(zshpower-shell prompt)"
    RPROMPT="$(zshpower-shell rprompt)"
}}
autoload add-zsh-hook
add-zsh-hook precmd set_zshpower
"""



from snakypy import printer
from snakypy.ansi import FG
from zshpower.config.base import Base
from zshpower.utils.process import reload_zsh
from zshpower.utils.check import checking_init
from zshpower.utils.shift import create_config
from zshpower.config.config import content as config_content


class ResetCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def main(self):
        checking_init(self.HOME)
        create_config(config_content, self.config_file, force=True)
        printer("Reset process finished.", foreground=FG.FINISH)
        reload_zsh()



from zshpower import __version__
from os.path import join
from zshpower.config import package


class Base:
    def __init__(self, home):
        self.HOME = home
        self.config_root = join(self.HOME, f".config/snakypy/zshpower/{__version__}")
        self.config_file = join(self.config_root, "config.toml")
        self.zsh_rc = join(self.HOME, ".zshrc")
        self.omz_root = join(self.HOME, ".oh-my-zsh")
        self.themes_folder = join(self.omz_root, "custom/themes")
        self.theme_file = join(
            self.themes_folder, f"{package.info['pkg_name']}.zsh-theme"
        )
        self.plugins = ("zsh-syntax-highlighting", "zsh-autosuggestions")



from sys import version_info as sys_version_info
from os import environ as os_environ, getcwd as os_getcwd
from os.path import exists, join
from .lib.utils import Color, symbol_ssh, separator, element_spacing


class Python(Color):
    def __init__(self, config):
        super().__init__()
        self.config = config
        self.search_f = (
            "__pycache__",
            "manage.py",
            "setup.py",
            "__init__.py",
            ".python-version",
            "requirements.txt",
            "pyproject.toml",
        )
        self.py_symbol = config["python"]["symbol"]
        self.py_symbol = symbol_ssh(config["python"]["symbol"], "py-")
        self.py_color = config["python"]["color"]
        self.py_prefix_color = config["python"]["prefix"]["color"]
        self.py_prefix_text = element_spacing(config["python"]["prefix"]["text"])
        self.py_version_enable = config["python"]["version"]["enable"]
        self.pyv_micro_enable = config["python"]["version"]["micro"]["enable"]

    def get_version(self, space_elem=" "):
        if not self.pyv_micro_enable:
            version = "{0[0]}.{0[1]}".format(sys_version_info)
            return f"{version}{space_elem}"
        else:
            version = "{0[0]}.{0[1]}.{0[2]}".format(sys_version_info)
            return f"{version}{space_elem}"

    def __str__(self):
        py_prefix1 = f"{Color(self.py_prefix_color)}{self.py_prefix_text}{Color().NONE}"
        if self.py_version_enable:
            for item in self.search_f:
                if exists(join(os_getcwd(), item)) or "VIRTUAL_ENV" in os_environ:
                    return str(
                        (
                            f"{separator(self.config)}{py_prefix1}"
                            f"{Color(self.py_color)}{self.py_symbol}"
                            f"{self.get_version()}{Color().NONE}"
                        )
                    )
        return ""



from os import geteuid as os_geteuid
from os import environ
from pathlib import Path
from .lib.utils import Color, abspath_link
from .lib.utils import symbol_ssh, element_spacing


def shorten_path(file_path, length):
    return Path(*Path(file_path).parts[-length:])


class Directory(Color):
    def __init__(self, config):
        super().__init__()
        self.username_enable = config["username"]["enable"]
        self.hostname_enable = config["hostname"]["enable"]
        self.directory_truncate_value = config["directory"]["truncation_length"]
        self.directory_symbol = symbol_ssh(config["directory"]["symbol"], "")
        self.directory_color = config["directory"]["color"]
        self.directory_prefix_color = config["directory"]["prefix"]["color"]
        self.directory_prefix_text = element_spacing(
            config["directory"]["prefix"]["text"]
        )

    def __str__(self, prefix="", space_elem=" "):
        if (
            self.username_enable
            or os_geteuid() == 0
            or self.hostname_enable
            or "SSH_CONNECTION" in environ
        ):
            prefix = (
                f"{Color(self.directory_prefix_color)}"
                f"{self.directory_prefix_text}{Color().NONE}"
            )

        directory = shorten_path(abspath_link(), int(self.directory_truncate_value))
        if (
            str(directory) == str(Path.home())
            or str(directory) == str(Path.home())[1:]
            # TODO: Error in user "root". Bugfix.
            # or str(directory) == str(Path.home()).split("/")[2].strip()
        ):
            directory = "~"

        directory_export = (
            f"{prefix}{Color(self.directory_color)}{self.directory_symbol}"
            f"{directory}{space_elem}{Color().NONE}"
        )

        return str(directory_export)



from datetime import datetime
from zshpower import __version__

content = f"""# Generate by: ZSHPower - D{datetime.today().isoformat()}
# Version: {__version__}
# ---------------------------------------------------------------------
# For more information, see the documentation at:
# https://github.com/snakypy/zshpower#configuration-file

[general]
jump_line.enable = true
config.editor = "vim"
separator.element = "-"
separator.color = "white"
position = ["package", "virtualenv", "python", "git"]

[username]
enable = false
color = "cyan"

[hostname]
enable = false
color = "magenta"
prefix.color = "green"
prefix.text = "at"

[directory]
truncation_length = 2
symbol = "\\ufc6e"
color = "cyan"
prefix.color = "green"
prefix.text = "in"

[git]
enable = true
symbol = "\\uf418"
branch.color = "cyan"
color.symbol = "white"
prefix.color = "green"
prefix.text = "on"

[git.status]
symbols.enable = true
symbol.clean = "\\uf62c"
symbol.added = "\\ufc03"
symbol.modified = "\\ufc07"
symbol.deleted = "\\ufbc7"
symbol.renamed = "\\uf101"
symbol.unmerged = "\\uf6fb"
symbol.untracked = "\\uf41e"
symbol.copied = "\\ufab1"
symbol.ahead = "\\uf55c"
symbol.behind = "\\uf544"
symbol.diverged = "\\ufb15"
symbol.conflicts = "\\uf0e7"

[command]
new_line.enable = true
symbol = "\\uf553"
color = "green"

[pyproject]
enable = true
symbol = "\\uf8d6"
color = "red"
prefix.color = "green"
prefix.text = "on"

[python]
symbol = "\\uf81f"
color = "yellow"
prefix.color = "green"
prefix.text = "via"
version.enable = true
version.micro.enable = true

[virtualenv]
enable = true
symbol = "\\ufcc2"
involved = "()"
color = "yellow"
prefix.color = "green"
prefix.text = "via"

[virtualenv.name]
normal.enable = true
text = "venv"

[timer]
enable = true
symbol = "\\uf43a"
color = "blue"
seconds.enable = false
"""



from socket import gethostname as socket_gethostname
from os import environ as os_environ
from .lib.utils import Color, element_spacing


class Hostname(Color):
    def __init__(self, config):
        super().__init__()
        self.config = config
        self.hostname_enable = self.config["hostname"]["enable"]
        self.hostname_color = self.config["hostname"]["color"]
        self.hostname_prefix_color = self.config["hostname"]["prefix"]["color"]
        self.hostname_prefix_text = element_spacing(
            self.config["hostname"]["prefix"]["text"]
        )

    def __str__(self, prefix="", space_elem=" "):
        if self.config["username"]["enable"] or "SSH_CONNECTION" in os_environ:
            prefix = (
                f"{Color(self.hostname_prefix_color)}"
                f"{self.hostname_prefix_text}"
                f"{Color().NONE}"
            )

        if "SSH_CONNECTION" in os_environ or self.hostname_enable:
            hostname_export = (
                f"{prefix}{Color(self.hostname_color)}"
                f"{socket_gethostname()}{space_elem}{Color().NONE}"
            )
            return str(hostname_export)

        return ""



from snakypy import printer
from snakypy.ansi import FG
from snakypy.console import billboard, credence
from zshpower.config import package
from zshpower import __version__


class CreditsCommand:
    @staticmethod
    def main():
        print("\n")
        printer("Offered by:".center(50), foreground=FG.GREEN)
        billboard(
            package.info["organization_name"], justify="center", foreground=FG.YELLOW
        )
        printer(f"copyright (c) since 2020\n".center(100), foreground=FG.GREEN)
        credence(
            package.info["name"],
            __version__,
            package.info["home_page"],
            package.info,
            foreground=FG.CYAN,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zshpower',
 'zshpower.commands',
 'zshpower.config',
 'zshpower.prompt',
 'zshpower.prompt.sections',
 'zshpower.prompt.sections.lib',
 'zshpower.utils']

package_data = \
{'': ['*']}

install_requires = \
['docopt>=0.6.2,<0.7.0',
 'fastentrypoints>=0.12,<0.13',
 'snakypy>=0.3.6,<0.4.0',
 'tomlkit>=0.5.11,<0.6.0']

entry_points = \
{'console_scripts': ['zshpower = zshpower.cli:main',
                     'zshpower-shell = zshpower.shell:main']}

setup_kwargs = {
    'name': 'zshpower',
    'version': '0.3.0',
    'description': 'ZSHPower is a theme for ZSH with a manager.',
    'long_description': '.. image:: https://raw.githubusercontent.com/snakypy/snakypy-static/master/zshpower/logo/png/zshpower.png\n    :width: 505 px\n    :align: center\n    :alt: ZSHPower\n\n.. image:: https://github.com/snakypy/zshpower/workflows/Python%20package/badge.svg\n    :target: https://github.com/snakypy/zshpower\n\n.. image:: https://img.shields.io/pypi/v/zshpower.svg\n    :target: https://pypi.python.org/pypi/zshpower\n\n.. image:: https://travis-ci.com/snakypy/zshpower.svg?branch=master\n    :target: https://travis-ci.com/snakypy/zshpower\n\n.. image:: https://img.shields.io/pypi/wheel/zshpower\n    :alt: PyPI - Wheel\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/psf/black\n\n.. image:: https://pyup.io/repos/github/snakypy/zshpower/shield.svg\n    :target: https://pyup.io/repos/github/snakypy/zshpower/\n    :alt: Updates\n\n.. image:: https://img.shields.io/github/issues-raw/snakypy/zshpower\n    :alt: GitHub issues\n\n.. image:: https://img.shields.io/github/license/snakypy/zshpower\n    :alt: GitHub license\n    :target: https://github.com/snakypy/zshpower/blob/master/LICENSE\n\n\n`ZSHPower` is a theme for `Oh My Zsh`_ framework; especially\nfor the `Python`_ developer. Pleasant to look at, the **ZSHPower**\ncomforts you with its colors and icons vibrant.\n\nInstalling **ZSHPower** is the easiest thing you will see in any existing theme for Oh My Zsh,\nbecause ZSHPower has its own manager. With `ZSHPower`_,\nchanges to the theme become more dynamic through a configuration file, where the user can make various combinations\nof changes, such as: **enable**, **disable**, **open settings**, **reset settings**, **uninstall** and more;\nall in a simplified command line, without opening any files or creating symbolic links.\nIn addition, the **ZSHPower** manager downloads **Oh My Zsh** and the\n`zsh-autosuggestions`_ and `zsh-syntax-highlighting`_ plugins automatically, everything to make your ZSH very power.\n\n\nRequirements\n------------\n\nTo work correctly, you will first need:\n\n* `git`_ (v2.25 or recent) must be installed.\n* `zsh`_  (v5.2 or recent) must be installed.\n* `python`_ (v3.7 or recent) must be installed.\n* `pip`_ (v19.3 or recent) must be installed.\n* `nerd fonts`_ must be installed.\n\n\nFeatures\n--------\n\n* `Oh My Zsh`_ Installation Automatically;\n* Automatically install `zsh-autosuggestions`_ and `zsh-syntax-highlighting`_;\n* Automated installation and uninstallation;\n* Enable and disable `ZSHPower` anytime;\n* Upgrade `ZSHPower` effortlessly;\n* Reset the settings with one command only;\n* Current Git branch and rich repo status:\n    *  — untracked changes;\n    *  — new files added;\n    *  — deleted files;\n    *  — new modified files;\n    *  — commits made;\n* Python version shown (*with pyenv support*) on the active virtual machine (E.g: `[python_icon] py-3.x`);\n* Shows the version of the project if you use "**pyproject.toml**" (E.g: `[pkg_icon] 0.1.0`);\n* Enables **username** and **hostname** when connecting with SSH. (can change in the settings to show permanently);\n* and, many other dynamic settings in `$HOME/.config/snakypy/zshpower/config.toml`.\n\n\nInstalling\n----------\n\nGlobally:\n\n.. code-block:: shell\n\n    $ sudo pip install zshpower\n\nFor the user:\n\n.. code-block:: shell\n\n    $ pip install zshpower --user\n\n\nUsing\n-----\n\nBecause **ZSHPower** is a manager, usage information is in the\n`ZSHPower`_ project. Access the project, and see how to use **ZSHPower**.\n\nFor more command information, run:\n\n.. code-block:: shell\n\n    $ zshpower --help\n\nMore information: https://github.com/snakypy/zshpower\n\nDonation\n--------\n\nIf you liked my work, buy me a coffee <3\n\n.. image:: https://www.paypalobjects.com/en_US/i/btn/btn_donateCC_LG.gif\n    :target: https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=YBK2HEEYG8V5W&source\n\nLicense\n-------\n\nThe gem is available as open source under the terms of the `MIT License`_ ©\n\nCredits\n-------\n\n* Name: William C. Canin\n* Country: Brazil - SP\n* E-Mail: william.costa.canin@gmail.com\n* Personal page: `William Canin`_\n\nThis package was created with Cookiecutter_ and the `williamcanin/pypkg-cookiecutter`_ project template.\n\nLinks\n-----\n\n* Code: https://github.com/snakypy/zshpower\n* Documentation: https://github.com/snakypy/zshpower/blob/master/README.md\n* Releases: https://pypi.org/project/zshpower/#history\n* Issue tracker: https://github.com/snakypy/zshpower/issues\n\n.. _Oh My Zsh: https://ohmyz.sh\n.. _Python: https://python.org\n.. _zsh-autosuggestions: https://github.com/zsh-users/zsh-autosuggestions\n.. _zsh-syntax-highlighting: https://github.com/zsh-users/zsh-syntax-highlighting\n.. _ZSHPower: https://github.com/snakypy/zshpower\n.. _git: https://git-scm.com/downloads\n.. _zsh: http://www.zsh.org/\n.. _`python`: https://python.org\n.. _pip: https://pip.pypa.io/en/stable/quickstart/\n.. _nerd fonts: https://www.nerdfonts.com/font-downloads\n.. _MIT License: https://github.com/snakypy/zshpower/blob/master/LICENSE\n.. _William Canin: http://williamcanin.github.io\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`williamcanin/pypkg-cookiecutter`: https://github.com/williamcanin/pypkg-cookiecutter\n',
    'author': 'William C. Canin',
    'author_email': 'william.costa.canin@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/snakypy/zshpower',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from os import popen as os_popen, getuid as os_getuid
from snakypy import printer
from re import search as re_search
from zshpower.config import package
from zshpower import __version__
from re import M as re_m
from snakypy.ansi import FG, NONE
from docopt import docopt
from zshpower.config import menu

try:
    import pwd
except ImportError:
    pass


def read_zshrc(zshrc):
    try:
        with open(zshrc) as f:
            return f.read()
    except FileNotFoundError as fnf_err:
        printer(f"File not found {fnf_err}", foreground=FG.ERROR)


def arguments(argv=None):
    formatted_version = f"{package.info['name']} version: {FG.CYAN}{__version__}{NONE}"
    data = docopt(menu.options, argv=argv, version=formatted_version)
    return data


def read_zshrc_omz(zshrc):
    """ """
    current_zshrc = read_zshrc(zshrc)
    m = re_search(r"ZSH_THEME=\".*", current_zshrc)
    if m is not None:
        var_zsh_theme = m.group(0)
        lst = var_zsh_theme.split("=")
        theme_name = [s.strip('"') for s in lst][1]
        return theme_name, var_zsh_theme
    return


def current_shell():
    pw = pwd.getpwuid(os_getuid())
    path_shell = pw[-1]
    shell = str(path_shell).split("/")[-1]
    return shell, path_shell


def current_user():
    return str(os_popen("whoami").read()).replace("\n", "")


def plugins_current_zshrc(zshrc):
    current_zshrc = read_zshrc(zshrc)
    m = re_search(r"^plugins=\(.*", current_zshrc, flags=re_m)
    if m is not None:
        get = m.group(0)
        lst = get.split("=")
        current = [i.strip('"').replace("(", "").replace(")", "") for i in lst][1]
        return current.split()
    return


def get_line_source(zshrc):
    """ """
    current_zshrc = read_zshrc(zshrc)
    m = re_search(r"source \$HOME/.zshpower", current_zshrc)
    if m is not None:
        return m.group(0)
    return



from pydoc import pager as pydoc_pager
from shutil import which as shutil_which
from zshpower.config.base import Base
from snakypy.file import read as snakypy_file_red
from tomlkit import parse as toml_parse
from os import environ
from subprocess import call as subprocess_call
from snakypy.file import read as snakypy_file_read
from zshpower.utils.check import checking_init


def editor_run(editor, config):
    if shutil_which(editor):
        get_editor = environ.get("EDITOR", editor)
        with open(config) as f:
            subprocess_call([get_editor, f.name])
            exit(0)
    return


class ConfigCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def main(self, arguments):

        checking_init(self.HOME)

        if arguments["--open"]:
            try:
                read_conf = snakypy_file_red(self.config_file)
                parsed = toml_parse(read_conf)
                editor_conf = parsed["general"]["config"]["editor"]
            except (FileNotFoundError, Exception):
                editor_conf = None

            if editor_conf:
                editor_run(editor_conf, self.config_file)
            else:
                editors = ("vim", "nano", "emacs", "micro")
                for edt in editors:
                    editor_run(edt, self.config_file)
        elif arguments["--view"]:
            read_config = snakypy_file_read(self.config_file)
            pydoc_pager(read_config)
            return True



from snakypy import printer
from zshpower.config import package
from os.path import join as os_join, isfile
from snakypy.ansi import FG, NONE
from zshpower.config.base import Base
from snakypy.path import create as snakypy_path_create
from snakypy.file import create as snakypy_file_create
from zshpower.utils.check import tools_requirements
from zshpower.utils.catch import get_line_source
from zshpower.config.config import content as config_content
from zshpower.config.zshrc import content as zshrc_content
from zshpower.config.set_zshpower import content as set_zshpower_content
from zshpower.utils.process import change_shell, reload_zsh
from zshpower.utils.shift import (
    create_config,
    omz_install,
    omz_install_plugins,
    install_fonts,
    create_zshrc,
    change_theme_in_zshrc,
    add_plugins_zshrc,
)

instruction_not_omz = f"""{FG.YELLOW}
********************** WARNING **********************
Add the following code to the {FG.MAGENTA}$HOME/.zshrc{NONE} {FG.YELLOW}file:

CODE: {FG.CYAN}source $HOME/.zshpower {NONE}
{FG.YELLOW}*****************************************************{NONE}
"""


class InitCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def main(self, arguments, *, reload=False, message=False):

        tools_requirements("git", "vim", "zsh")

        snakypy_path_create(self.config_root)

        create_config(config_content, self.config_file)

        snakypy_file_create(
            set_zshpower_content,
            os_join(self.HOME, f".{package.info['pkg_name']}"),
            force=True,
        )

        if arguments["--omz"]:
            omz_install(self.omz_root)
            omz_install_plugins(self.omz_root, self.plugins)
            create_zshrc(zshrc_content, self.zsh_rc)
            change_theme_in_zshrc(self.zsh_rc, f"{package.info['pkg_name']}")
            add_plugins_zshrc(self.zsh_rc)
            snakypy_file_create(set_zshpower_content, self.theme_file, force=True)

        install_fonts(self.HOME)

        change_shell()

        printer("Done!", foreground=FG.FINISH) if message else None

        if not arguments["--omz"] and not get_line_source(self.zsh_rc):
            printer(instruction_not_omz, foreground=FG.YELLOW)

        reload_zsh() if reload else None



"""
ZSHPower
~~~~~~~~

ZSHPower is a theme for ZSH with a manager.


For more information, access: 'https://github.com/snakypy/zshpower'

:copyright: Copyright 2020-2020 by Snakypy team, see AUTHORS.
:license: MIT license, see LICENSE for details.
"""

from pathlib import Path

HOME = str(Path.home())

__version__ = "0.3.0"



from sys import argv as sys_argv, stdout
from tomlkit import parse as toml_parse
from snakypy.file import read as snakypy_file_red
from snakypy.utils.decorators import only_for_linux
from snakypy.path import create as snakypy_path_create
from zshpower import HOME
from zshpower.utils.shift import create_config
from zshpower.config.config import content as config_content
from zshpower.prompt.sections.command import Command
from zshpower.prompt.sections.directory import Directory
from zshpower.prompt.sections.git import Git
from zshpower.prompt.sections.hostname import Hostname
from zshpower.prompt.sections.package import PyProject
from zshpower.prompt.sections.python import Python
from zshpower.prompt.sections.timer import Timer
from zshpower.prompt.sections.username import Username
from zshpower.prompt.sections.virtualenv import Virtualenv
from zshpower.config.base import Base


class Prompt(Base):
    """Class to perform the impression of the PROMPT style"""

    def __init__(self):
        Base.__init__(self, HOME)

    @property
    def config_load(self):
        try:
            read_conf = snakypy_file_red(self.config_file)
            parsed = toml_parse(read_conf)
            return parsed
        except FileNotFoundError:
            snakypy_path_create(self.config_root)
            create_config(config_content, self.config_file)
            read_conf = snakypy_file_red(self.config_file)
            parsed = toml_parse(read_conf)
            # printer(
            #     f"[ZSHPower Warning] A new configuration file for that version "
            #     f'has been created in "{self.config_root}".',
            #     foreground=FG.YELLOW,
            # )
            return parsed

    def left(self, jump_line="\n"):
        if not self.config_load["general"]["jump_line"]["enable"]:
            jump_line = ""
        username = Username(self.config_load)
        hostname = Hostname(self.config_load)
        directory = Directory(self.config_load)
        dinamic_section = {
            "package": PyProject(self.config_load),
            "python": Python(self.config_load),
            "virtualenv": Virtualenv(self.config_load),
            "git": Git(self.config_load),
        }
        cmd = Command(self.config_load)

        static_section = f"{jump_line}{username}{hostname}{directory}"

        ordered_section = []
        for element in self.config_load["general"]["position"]:
            for item in dinamic_section.keys():
                if item == element:
                    # stdout.write(str(dinamic_section[item]))
                    ordered_section.append(dinamic_section[item])
        sections = "{}{}{}{}{}{}"
        return sections.format(static_section, *ordered_section, cmd)

    def right(self):
        timer = str(Timer(self.config_load))
        return timer


@only_for_linux
def main():
    if len(sys_argv) >= 1 and sys_argv[1] == "prompt":
        stdout.write(Prompt().left())
    elif len(sys_argv) >= 1 and sys_argv[1] == "rprompt":
        stdout.write(Prompt().right())



from shutil import which
from os.path import join, exists
from snakypy import printer, FG
from zshpower.config import package


def tools_requirements(*args):
    for tool in args:
        if which(tool) is None:
            raise Exception(
                f'The package \'{package.info["name"]}\' needs the "{tool}" tool.'
                " Tool not found. Aborted."
            )


def checking_init(home):
    """Function that ends commands that depend on the created repository, but
    the repository was not created."""
    if not exists(join(home, f".{package.info['pkg_name']}")):
        printer(
            f'Command "{package.info["pkg_name"]} init" has not been started.'
            "Aborted",
            foreground=FG.WARNING,
        )
        exit(1)
    return True



from tomlkit import parse as toml_parse
from snakypy.file import read as snakypy_file_read
from os import getcwd
from os.path import isfile, join
from .lib.utils import Color, symbol_ssh, separator, element_spacing


class PyProject(Color):
    def __init__(self, config):
        super().__init__()
        self.config = config
        self.pyproject_f = join(getcwd(), "pyproject.toml")
        self.pyproject_enable = config["pyproject"]["enable"]
        self.pyproject_symbol = symbol_ssh(config["pyproject"]["symbol"], "pkg-")
        self.pyproject_color = config["pyproject"]["color"]
        self.pyproject_prefix_color = config["pyproject"]["prefix"]["color"]
        self.pyproject_prefix_text = element_spacing(
            config["pyproject"]["prefix"]["text"]
        )

    def get_version(self, space_elem=" "):
        if isfile(self.pyproject_f):
            read_f = snakypy_file_read(self.pyproject_f)
            parsed = dict(toml_parse(read_f))
            for item in parsed.values():
                for data in item.values():
                    return f"{data['version']}{space_elem}"
        return ""

    def __str__(self):
        if self.pyproject_enable and isfile(self.pyproject_f):
            pyproject_prefix = (
                f"{Color(self.pyproject_prefix_color)}"
                f"{self.pyproject_prefix_text}{Color().NONE}"
            )
            pyproject_export = (
                f"{separator(self.config)}{pyproject_prefix}"
                f"{Color(self.pyproject_color)}"
                f"{self.pyproject_symbol}{self.get_version()}{Color().NONE}"
            )
            return str(pyproject_export)
        return ""
